% Berechnet die Partialsummen der geometrischen Reihen mit den
% verschiedenen Codes aus Aufgabe 20 (siehe geomSumA20.m und
% geomSumA20optimized.m).

q=0.9;

% Zunächst die ineffiziente Implementierung:
N1=1000:1000:20000; % Berechne Partialsummen mit N=1000,2000,...,20000
times1=zeros(length(N1),1); % Vektor, in dem die Rechenzeiten gespeichert werden

for k=1:length(N1)
    tic % Starte Stoppuhr
    S=geomSumA20(q,N1(k));
    times1(k)=toc; % Beende Stoppuhr und speichere Zeit im Vektor times
end

% Nun für die effiziente Implementierung. Hier sind größere Werte für N
% notwendig, um den Effekt sichtbar zu machen
N2=10000:10000:200000;
times2=zeros(length(N2),1);
for k=1:length(N2)
    tic
    S=geomSumA20optimized(q,N2(k));
    times2(k)=toc;
end


subplot(2,1,1);
plot(N1,times1);
subplot(2,1,2);
plot(N2,times2);

% Betrachtet man die beiden Plots, fällt auf:
% Der erste Algorithmus braucht sehr viel länger als der zweite
% Beim zweiten Algorithmus wächst die Rechenzeit linear in N, beim ersten
% hingegen eher wie ein Polynom höheren Grades

% Dies liegt daran, dass beim optimierten Code in jedem Schleifendurchlauf
% nur 2 Rechenoperationen nötig sind, insgesamt also 2*N
% Im schlechten Code sind hingegen im k-ten Schleifendurchlauf k (+1)
% Operationen notwendig, um den Summand q^k zu berechnen. Für N
% Schleifendurchläufe braucht man demnach grob
% 1+2+3+...+N = N(N+1)/2 Operationen, d. h. der Aufand wächst im
% Wesentlichen quadratisch in N.